--Set color variables default values to avoid first load errors :M
cvred = 1
cvgreen = .996
cvblue = .545
cvalpha = 1

--Hard coded color options table :L
local colorTable = {
	["blue"] = {0, 0, 1},
	["green"] = {0, 1, 0},
	["red"] = {1, 0, 0},
	["black"] = {0, 0, 0},
	["white"] = {1, 1, 1},
	["lightblue"] = {0, 1, 1},
	["lightred"] = {1, .5, .5},
	["pink"] = {1, .5, 1},
	["purple"] = {.7, 0, 1},
	["orange"] = {1, 0.5, 1},
	["default"] = {1, .996, .545}
}

--Classes
local str_to_ap = {
	["DRUID"] = 2,
	["HUNTER"] = 1,
	["MAGE"] = 0,
	["PALADIN"] = 2,
	["PRIEST"] = 0,
	["ROGUE"] = 1,
	["SHAMAN"] = 2,
	["WARLOCK"] = 0,
	["WARRIOR"] = 2
}
local str_to_block = {
	["DRUID"] = 0,
	["HUNTER"] = 0,
	["MAGE"] = 0,
	["PALADIN"] = 20,
	["PRIEST"] = 0,
	["ROGUE"] = 0,
	["SHAMAN"] = 20,
	["WARLOCK"] = 0,
	["WARRIOR"] = 20
}

local agil_to_ap = {
	["DRUID"] = 1,
	["HUNTER"] = 2,
	["MAGE"] = 0,
	["PALADIN"] = 0,
	["PRIEST"] = 0,
	["ROGUE"] = 1,
	["SHAMAN"] = 0,
	["WARLOCK"] = 0,
	["WARRIOR"] = 1
}
local agil_to_crit = {
	["DRUID"] = 20,
	["HUNTER"] = 53,
	["MAGE"] = 0,
	["PALADIN"] = 20,
	["PRIEST"] = 0,
	["ROGUE"] = 29,
	["SHAMAN"] = 20,
	["WARLOCK"] = 0,
	["WARRIOR"] = 20
}
local agil_to_dodge = {
	["DRUID"] = 20,
	["HUNTER"] = 26,
	["MAGE"] = 20,
	["PALADIN"] = 20,
	["PRIEST"] = 20,
	["ROGUE"] = 14.5,
	["SHAMAN"] = 20,
	["WARLOCK"] = 20,
	["WARRIOR"] = 20
}
local int_to_mana = {
	["DRUID"] = 15,
	["HUNTER"] = 15,
	["MAGE"] = 15,
	["PALADIN"] = 15,
	["PRIEST"] = 15,
	["ROGUE"] = 0,
	["SHAMAN"] = 15,
	["WARLOCK"] = 15,
	["WARRIOR"] = 0
}
local int_to_crit = {
	["DRUID"] = 60,
	["HUNTER"] = 0,
	["MAGE"] = 59.5,
	["PALADIN"] = 54,
	["PRIEST"] = 59.2,
	["ROGUE"] = 0,
	["SHAMAN"] = 59.5,
	["WARLOCK"] = 60.6,
	["WARRIOR"] = 0
}
local stam_to_health = {
	["DRUID"] = 10,
	["HUNTER"] = 10,
	["MAGE"] = 10,
	["PALADIN"] = 10,
	["PRIEST"] = 10,
	["ROGUE"] = 10,
	["SHAMAN"] = 10,
	["WARLOCK"] = 10,
	["WARRIOR"] = 10
}
local spirit_to_regen = {
	["DRUID"] = 15,
	["HUNTER"] = 15,
	["MAGE"] = 12.5,
	["PALADIN"] = 15,
	["PRIEST"] = 12.5,
	["ROGUE"] = 0,
	["SHAMAN"] = 17,
	["WARLOCK"] = 15,
	["WARRIOR"] = 0
}

local spirit_to_regen_class = {
	["DRUID"] = 4.5,
	["HUNTER"] = 5,
	["MAGE"] = 4,
	["PALADIN"] = 5,
	["PRIEST"] = 4,
	["ROGUE"] = 0,
	["SHAMAN"] = 5,
	["WARLOCK"] = 5,
	["WARRIOR"] = 0
}

--Color Picker
local function ShowColorPicker(cvred, cvgreen, cvblue, cvalpha, changedCallback)
 ColorPickerFrame:SetColorRGB(cvred, cvgreen, cvblue);
 ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (cvalpha ~= nil), cvalpha;
 ColorPickerFrame.previousValues = {cvred, cvgreen, cvblue, cvalpha};
 ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc =  changedCallback, changedCallback, changedCallback;
 ColorPickerFrame:Hide();
 ColorPickerFrame:Show();
end

local function myColorCallback(restore)
 local newR, newG, newB, newA;
 if restore then
  newR, newG, newB, newA = unpack(restore);  
 else
  newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB();
 end 
  cvred, cvgreen, cvblue, cvalpha = newR, newG, newB, newA;  
end

--Slash Command to change the color of the output :M
SLASH_CONVERTRATINGS1, SLASH_CONVERTRATINGS2 = '/convertratings', '/cvr';
function SlashCmdList.CONVERTRATINGS(msg, editBox)
	--Grab the first input word as the command and the rest of the input as a user variable :M
	local command, rest = msg:match("^(%S*)%s*(.-)$");
	--Hard coded color options parsing :M&L
	
	if (colorTable[string.lower(command)]) then
		cvred, cvgreen, cvblue = unpack(colorTable[string.lower(command)]);
		print("Convert Ratings output color set to "..string.lower(command));
	elseif string.lower(command) == 'custom' and rest == "" then
		ShowColorPicker(cvred, cvgreen, cvblue, nil, myColorCallback);
	else
		--when no valid args entered, output this stuff :M
		print("Convert Ratings: Valid color options are red, green, blue, black, white, lightblue, lightred, pink, purple, orange or custom")
		print("Convert Ratings: The custom option will bring up the Color Picker for you to choose a color.")
		print("Convert Ratings: To reset to the default color, use /convertratings default")
	end
end

--rating tables to allow addon to work at all levels :M

--Create Function to round the decimals :M
local function mathround(number, precision)
  precision = precision or 0

  local decimal = string.find(tostring(number), ".", nil, true);
  
  if ( decimal ) then  
    local power = 10 ^ precision;
    
    if ( number >= 0 ) then 
      number = math.floor(number * power + 0.5) / power;
    else 
      number = math.ceil(number * power - 0.5) / power;    
    end
    
    -- convert number to string for formatting :M
    number = tostring(number);      
    
    -- set cutoff :M
    local cutoff = number:sub(decimal + 1 + precision);
      
    -- delete everything after the cutoff :M
    number = number:gsub(cutoff, "");
  else
    -- number is an integer :M
    if ( precision > 0 ) then
      number = tostring(number);
      
      number = number .. ".";
      
      for i = 1,precision
      do
        number = number .. "0";
      end
    end
  end    
  return number;
end

function dump(o)
	if type(o) == 'table' then
	   local s = '{ '
	   for k,v in pairs(o) do
		  if type(k) ~= 'number' then k = '"'..k..'"' end
		  s = s .. '['..k..'] = ' .. dump(v) .. ','
	   end
	   return s .. '} '
	else
	   return tostring(o)
	end
 end

 function isNumeric(value)
    if value == tostring(tonumber(value)) then
        return true
    else
        return false
    end
end

-- Classic Variables
local rawintellect, rawstrength, rawagility, rawspirit, rawstamina;
local cStrAP, cStrBLK, cAgilAP, cAgilCRIT, cAgilDODGE, cIntCRIT, cIntMANA, cStamHP, cSpiritREGEN;
local strAP, strBLK, agilAP, agilCRIT, agilDODGE, intCRIT, intMANA, stamHP, spiritREGEN, spiritREGENCLASS;

local function convertRawStats()
	--convert raw stats into percentages so long as they are not nil
	if rawstrength ~= nil then		
		if (strAP ~= 0) then
			cStrAP = rawstrength * strAP
		end
		if (strBLK ~= 0) then
			cStrBLK = rawstrength / strBLK
			cStrBLK = mathround(cStrBLK, 2)
		end
	end
	if rawagility ~= nil then
		if (agilCRIT ~= 0) then
			cAgilCRIT = rawagility / agilCRIT
			cAgilCRIT = mathround(cAgilCRIT, 2)
		end
		if (agilAP ~= 0) then
			cAgilAP = rawagility * agilAP
		end
		if (agilDODGE ~= 0) then
			cAgilDODGE = rawagility / agilDODGE;
			cAgilDODGE = mathround(cAgilDODGE, 2)
		end
	end
	if rawintellect ~= nil then
		if (intCRIT ~= 0) then
			cIntCRIT = rawintellect / intCRIT
			cIntCRIT = mathround(cIntCRIT, 2)
		end	
		if (intMANA ~= 0) then
			cIntMANA = rawintellect * intMANA
		end	
	end
	if rawstamina ~= nil then
		if (stamHP ~= nil) then
			cStamHP = rawstamina * stamHP
		end
	end
	if rawspirit ~= nil then
		if (spiritREGEN ~= 0) then
			cSpiritREGEN = (rawspirit / spiritREGENCLASS) + spiritREGEN
			cSpiritREGEN = mathround(cSpiritREGEN, 2)
		end
	end
end



--Here is the function where the stats are pulled from the item that is currently moused over
local function getItemIdFromTooltip(self)
		
   	--Get itemLink of mouseover :L
	local name, itemLink = self:GetItem();
	--Check to make sure an itemLink is actually returned
	if(itemLink == nil) then
		return;
	end

	--Set Player level :M
	cvlevel = UnitLevel("player")
	localizedClass, englishClass, classIndex = UnitClass("Player");
	
	--Select the level based rating amounts :M
	strAP = str_to_ap[englishClass];
	strBLK = str_to_block[englishClass];
	agilAP = agil_to_ap[englishClass];
	agilCRIT = agil_to_crit[englishClass];
	agilDODGE = agil_to_dodge[englishClass];
	intCRIT = int_to_crit[englishClass];
	intMANA = int_to_mana[englishClass];
	stamHP = stam_to_health[englishClass];
	spiritREGEN = spirit_to_regen[englishClass];
	spiritREGENCLASS = spirit_to_regen_class[englishClass];

	--Declare variables for future use :M
	local hexcolor;
	
	--Get Item rarity :M
	local irare = select(3,GetItemInfo(itemLink))
	local itemType = select(6,GetItemInfo(itemLink))
	
	if (itemType ~= "Armor" and itemType ~= "Weapon") then
		return;
	end

	--Gets stats from non Artifact items :M
	stats = GetItemStats(itemLink);
		
	--If not an item with stats, don't do anything :L
	if(stats == nil) then
		return;
	end
	
	rawstrength = stats["ITEM_MOD_STRENGTH_SHORT"]
	rawagility = stats["ITEM_MOD_AGILITY_SHORT"]
	rawintellect = stats["ITEM_MOD_INTELLECT_SHORT"]
	rawstamina = stats["ITEM_MOD_STAMINA_SHORT"]
	rawspirit = stats["ITEM_MOD_SPIRIT_SHORT"]
	
	convertRawStats()
    
	
    --Convert percentages to strings :M
    --tostring(pagil)
	
	--Convert text color from decimal to hex
	hexcolor = string.format("|cff%02x%02x%02x", cvred*255, cvgreen*255, cvblue*255)
	white = string.format("|cff%02x%02x%02x", 255, 255, 255)
	
	-- aggregate stats (crit+crit etc and add stats section?)

	-- find where to inject (copy previous lines, add new aggregate stats line) then add the rest

	-- add a line, where is it at..
	--self:AddLine("New tooltip line", 1, 1, 1)
	--_G[self:GetName().."TextLeft"..self:NumLines()]:SetText(white .. "+" .. rawagility .. "" .. _G["ITEM_MOD_AGILITY_SHORT"] .. hexcolor .. " (" .. pagil .. "% Crit)");
	--_G[self:GetName().."TextLeft"..toint(self:NumLines+1)]:SetText(white .. "+" .. rawstrength .. "" .. _G["ITEM_MOD_STRENGTH_SHORT"] .. hexcolor .. " (+" .. pstr .. " AP)");
	--_G[self:GetName().."TextLeft"..toint(self:NumLines+1)]:SetText(white .. "+" .. rawintellect .. "" .. _G["ITEM_MOD_INTELLECT_SHORT"] .. hexcolor .. " (" .. pint .. "% Crit)");

	--self:Show()

	--Set output values in the same line as the rating in tooltip :L
	for i=1, self:NumLines() do		
		local line = _G[self:GetName().."TextLeft"..i]
		local text = line:GetText()
		if text then


			-- because wow is dumb, and sometimes we need to correct it....
			if rawstrength == nil and string.find(text, _G["ITEM_MOD_STRENGTH_SHORT"]) then
				rawstrength = string.match(text,"%d+")
				convertRawStats()
			elseif rawagility == nil and string.find(text, _G["ITEM_MOD_AGILITY_SHORT"]) then
				rawagility = string.match(text,"%d+")
				convertRawStats()
			elseif rawintellect == nil and string.find(text, _G["ITEM_MOD_INTELLECT_SHORT"]) then
				rawintellect = string.match(text,"%d+")
				convertRawStats()
			elseif rawstamina == nil and string.find(text, _G["ITEM_MOD_STAMINA_SHORT"]) then
				rawstamina = string.match(text,"%d+")
				convertRawStats()
			elseif rawspirit == nil and string.find(text, _G["ITEM_MOD_SPIRIT_SHORT"]) then
				rawspirit = string.match(text,"%d+")
				convertRawStats()
			end

			--If line contains "Critical Strike", then sets show a 'fontString' and set its text :L
			local stringBuilder, stringSep;
			if(string.find(text, _G["ITEM_MOD_AGILITY_SHORT"])) and rawagility ~= nil then
				stringBuilder = " (";
				if (agilAP~=0) then
					stringBuilder = stringBuilder .. "+" .. cAgilAP .. " AP"
				end
				if (agilCRIT~=0) then
					if strlen(stringBuilder) > 3 then stringSep = ", " else stringSep = "" end
					stringBuilder = stringBuilder .. stringSep .. cAgilCRIT .. "% Crit"
				end
				if (agilDODGE~=0) then
					if strlen(stringBuilder) > 3 then stringSep = ", " else stringSep = "" end
					stringBuilder = stringBuilder .. stringSep .. cAgilDODGE .. "% Dodge"
				end
				stringBuilder = stringBuilder .. ")";

				if strlen(stringBuilder) > 3 then
					line:SetText(white .. "+" .. rawagility .. " " .. _G["ITEM_MOD_AGILITY_SHORT"] .. hexcolor .. "" .. stringBuilder);
				end
			end
			if(string.find(text, _G["ITEM_MOD_STRENGTH_SHORT"])) and rawstrength ~= nil then
				stringBuilder = " (";
				if (strAP~=0) then
					stringBuilder = stringBuilder .. "+" .. cStrAP .. " AP"
				end
				if (strBLK~=0) then
					if strlen(stringBuilder) > 3 then stringSep = ", +" else stringSep = "" end
					stringBuilder = stringBuilder .. stringSep .. cStrBLK .. " Block"
				end
				stringBuilder = stringBuilder .. ")";
				if strlen(stringBuilder) > 3 then
					line:SetText(white .. "+" .. rawstrength .. " " .. _G["ITEM_MOD_STRENGTH_SHORT"] .. hexcolor .. "" .. stringBuilder);
				end
			end
			if(string.find(text, _G["ITEM_MOD_INTELLECT_SHORT"])) and rawintellect ~= nil then
				
				stringBuilder = " (";
				if (intCRIT~=0) then
					stringBuilder = stringBuilder .. cIntCRIT .. "% Spell Crit"
				end	
				if (intMANA~=0) then
					if strlen(stringBuilder) > 3 then stringSep = ", +" else stringSep = "" end
					stringBuilder = stringBuilder .. stringSep .. cIntMANA .. " Mana"
				end
				stringBuilder = stringBuilder .. ")";
				if strlen(stringBuilder) > 3 then
					line:SetText(white .. "+" .. rawintellect .. " " .. _G["ITEM_MOD_INTELLECT_SHORT"] .. hexcolor .. "" .. stringBuilder);
				end
			end
			if(string.find(text, _G["ITEM_MOD_STAMINA_SHORT"])) and rawstamina ~= nil then
				stringBuilder = " (";
				if (stamHP~=0) then
					stringBuilder = stringBuilder .. "+" .. cStamHP .. " Health"
				end
				stringBuilder = stringBuilder .. ")";
				if strlen(stringBuilder) > 3 then
					line:SetText(white .. "+" .. rawstamina .. " " .. _G["ITEM_MOD_STAMINA_SHORT"] .. hexcolor .. "" .. stringBuilder);
				end
			end
			if(string.find(text, _G["ITEM_MOD_SPIRIT_SHORT"])) and rawspirit ~= nil then
				stringBuilder = " (";
				if (spiritREGEN~=0) then
					stringBuilder = stringBuilder .. "+" .. cSpiritREGEN .. " Mana Regen"
				end
				stringBuilder = stringBuilder .. ")";
				if strlen(stringBuilder) > 3 then
					line:SetText(white .. "+" .. rawspirit .. " " .. _G["ITEM_MOD_SPIRIT_SHORT"] .. hexcolor .. "" .. stringBuilder);
				end
			end

			
		end
		
	end

end

--Hooks to make the addon function :L
GameTooltip:HookScript("OnTooltipSetItem", getItemIdFromTooltip);
ItemRefTooltip:HookScript("OnTooltipSetItem", getItemIdFromTooltip);
ShoppingTooltip1:HookScript("OnTooltipSetItem", getItemIdFromTooltip);
ShoppingTooltip2:HookScript("OnTooltipSetItem", getItemIdFromTooltip);
WorldMapTooltip.ItemTooltip.Tooltip:HookScript("OnTooltipSetItem", getItemIdFromTooltip);